@@ -55,9 +55,7 @@ struct tlist {
   { CGDB_KEY_END,   "@7", NULL, "kend",   NULL },
   { CGDB_KEY_HOME,  "kh", NULL, "khome",  NULL },
   { CGDB_KEY_HOME,  "kH", NULL, "kll",    NULL },
-  { CGDB_KEY_DC,    "dc", NULL, "dch1",   NULL },
   { CGDB_KEY_DC,    "kD", NULL, "kdch1",  NULL },
-  { CGDB_KEY_IC,    "ic", NULL, "ich1",   NULL },
   { CGDB_KEY_IC,    "kI", NULL, "kich1",  NULL },
   { CGDB_KEY_NPAGE, "kN", NULL, "knp",    NULL },
   { CGDB_KEY_PPAGE, "kP", NULL, "kpp",    NULL },
@@ -784,7 +782,7 @@ kui_term_get_ascii_char_sequence_from_key (int key)
   int i, val;
   const char *keycode;
 
-  if (key <= CGDB_KEY_ESC || key >= CGDB_KEY_ERROR)
+  if (key < CGDB_KEY_ESC || key >= CGDB_KEY_ERROR)
     return NULL;
 
   for (i = 0; hard_coded_bindings[i].key != CGDB_KEY_ERROR; ++i)
