@@ -110,6 +110,29 @@ struct kui_manager *kui_ctx = NULL;	/* The key input package */
 struct kui_map_set *kui_map = NULL;
 struct kui_map_set *kui_imap = NULL;
 
+/**
+ * This allows CGDB to know if it is acceptable to read more input from
+ * the KUI at this particular moment. Under certain circumstances, CGDB may 
+ * have to complete a particular communication message to GDB, in while doing
+ * so, may not want to process's the users keystrokes. This flag is purely for
+ * CGDB to keep track of if it wants to read from the KUI. The KUI is always
+ * ready.
+ *
+ * If kui_input_acceptable is set to 1, then input can be read from
+ * the kui. Otherwise, some part of CGDB is waiting for more events to happen.
+ *
+ * For example, when the user types 'o' at the CGDB source window, the
+ * user is requesting the file dialog to open. CGDB must first ask GDB for 
+ * the list of files. While it is retreiving these files, CGDB shouldn't 
+ * shoot through the rest of the kui keys available. If the user had
+ * typed 'o /main' they would want to open the file dialog and start 
+ * searching for a file that had the substring 'main' in it. So, CGDB must
+ * first ensure all the files are retrieved, displayed in the file
+ * dialog, and ensure the file dialog is ready to recieve keys from the 
+ * user before the input is allowed to hit the file dialog.
+ */
+int kui_input_acceptable = 1;
+
 int resize_pipe[2] = { -1, -1 };
 
 /* Readline interface */
@@ -298,16 +321,14 @@ rlctx_send_user_command (char *line)
   tgdb_request_ptr request_ptr;
   int i;
 
-  /* This happens when rl_callback_read_char gets EOF */
-  if (line == NULL)
-  {
-    cleanup ();
-    exit (0);
+  if (line == NULL) {
+    /* NULL line means rl_callback_read_char received EOF */
+    ibuf_add(current_line, "quit");
+  } else {
+    /* Add the line passed in to the current line */
+    ibuf_add (current_line, line);
   }
 
-  /* Add the line passed in to the current line */
-  ibuf_add (current_line, line);
-
   /* Get current line, and current line length */
   cline = ibuf_get (current_line);
   length = ibuf_length (current_line);
@@ -668,7 +689,7 @@ version_info (void)
 {
   static char buf[MAXLINE];
   sprintf (buf, "%s %s\r\n%s", "CGDB", VERSION,
-	   "Copyright 2002-2007 Bob Rossi and Mike Mueller.\n"
+	   "Copyright 2002-2010 Bob Rossi and Mike Mueller.\n"
 	   "CGDB is free software, covered by the GNU General Public License, and you are\n"
 	   "welcome to change it and/or distribute copies of it under certain conditions.\n"
 	   "There is absolutely no warranty for CGDB.\n");
@@ -893,7 +914,13 @@ user_input (void)
 }
 
 /**
- * This will process all the input that the KUI has.
+ * This will usually process all the input that the KUI has.
+ *
+ * However, it's possible that one of the keys the user sends to CGDB 
+ * switches the state of CGDB in such a way, that CGDB has to do some I/O
+ * with GDB before the keys can continue to be sent to CGDB. For this reason,
+ * this loop can return before all the KUI's data has been used, in order to
+ * give the main loop a chance to run a GDB command.
  *
  * \return
  * 0 on success or -1 on error
@@ -902,6 +929,11 @@ static int
 user_input_loop ()
 {
   do {
+    /* There are reasons that CGDB should wait to get more info from the kui.
+     * See the documentation for kui_input_acceptable */
+    if (!kui_input_acceptable)
+      return 0;
+
     if (user_input () == -1) {
       logger_write_pos (logger, __FILE__, __LINE__, "user_input_loop failed");
       return -1;
@@ -971,7 +1003,7 @@ process_commands (struct tgdb *tgdb)
 
 	  /* This is a list of all the source files */
 	case TGDB_UPDATE_SOURCE_FILES:
-	  {
+	  { 
 	    struct tgdb_list *list =
 	      item->choice.update_source_files.source_files;
 	    tgdb_list_iterator *i = tgdb_list_get_first (list);
@@ -987,6 +1019,7 @@ process_commands (struct tgdb *tgdb)
 	      }
 
 	    if_set_focus (FILE_DLG);
+            kui_input_acceptable = 1;
 	    break;
 	  }
 
@@ -996,6 +1029,7 @@ process_commands (struct tgdb *tgdb)
 	case TGDB_SOURCES_DENIED:
 	  if_display_message ("Error:", 0,
 			      " No sources available! Was the program compiled with debug?");
+     kui_input_acceptable = 1;
 	  break;
 
 	  /* This is the absolute path to the last file the user requested */
@@ -1337,10 +1371,8 @@ main_loop (void)
 
   for (;;)
     {
-
       /* Reset the fd_set, and watch for input from GDB or stdin */
       FD_ZERO (&rset);
-
       FD_SET (STDIN_FILENO, &rset);
       FD_SET (gdb_fd, &rset);
       FD_SET (tty_fd, &rset);
@@ -1405,9 +1437,21 @@ main_loop (void)
 	}
 
       /* gdb's output -> stdout */
-      if (FD_ISSET (gdb_fd, &rset))
-	if (gdb_input () == -1)
+      if (FD_ISSET (gdb_fd, &rset)) {
+	if (gdb_input () == -1) {
 	  return -1;
+        }
+
+        /* When the file dialog is opened, the user input is blocked, 
+         * until GDB returns all the files that should be displayed,
+         * and the file dialog can open, and be prepared to receive 
+         * input. So, if we are in the file dialog, and are no longer
+         * waiting for the gdb command, then read the input.
+         */
+        if (kui_manager_cangetkey (kui_ctx)) {
+            user_input_loop ();
+        }
+      }
 
       /* A resize signal occured */
       if (FD_ISSET (resize_pipe[0], &rset))
@@ -1552,6 +1596,37 @@ update_kui (cgdbrc_config_option_ptr option)
   return 0;
 }
 
+static int
+destroyReadlineKeySeq (void *data)
+{
+   char *keyseq = (char*)data;
+   free (keyseq);
+   return 0;
+}
+
+int
+add_readline_key_sequence (const char *readline_str, enum cgdb_key key)
+{
+  int ret_val;
+
+  std_list keyseq_list = std_list_create (destroyReadlineKeySeq);
+
+  ret_val = rline_get_keyseq (rline, readline_str, keyseq_list);
+  if (ret_val == -1) {
+     std_list_destroy (keyseq_list);
+     return -1;
+  }
+
+  ret_val = kui_manager_get_terminal_keys_kui_map (kui_ctx, key, keyseq_list);
+  if (ret_val == -1) {
+     std_list_destroy (keyseq_list);
+     return -1;
+  }
+  std_list_destroy (keyseq_list);
+
+  return 0;
+}
+
 int
 init_kui (void)
 {
@@ -1599,6 +1674,26 @@ init_kui (void)
   cgdbrc_attach (CGDBRC_TTIMEOUT, &update_kui, NULL);
   cgdbrc_attach (CGDBRC_TTIMEOUT_LEN, &update_kui, NULL);
 
+  /* It's important that CGDB uses readline's view of 
+   * Home and End keys. A few distros I've run into (redhat e3
+   * and ubuntu) provide incorrect terminfo entries for xterm.
+   * So, Home and End do not work. The distro's fixed readline
+   * by modifing /etc/inputrc to hard code the terminal sequences.
+   * I have no idea why they wouldn't just fix the terminfo 
+   * database, but they didn't! Therefor, readline, bash, gdb all
+   * work but cgdb doesn't. So, I'm going to simply ask readline
+   * what it thinks the Home and End keys are and add them to 
+   * CGDB's mappings.
+   */
+
+  /* For now, I've decided it's OK for these functions to fail as they
+   * only add functionality to CGDB. */
+  
+  /* Home key */
+  add_readline_key_sequence ("beginning-of-line", CGDB_KEY_HOME);
+  /* End key */
+  add_readline_key_sequence ("end-of-line", CGDB_KEY_END);
+
   return 0;
 }
 
@@ -1615,6 +1710,8 @@ main (int argc, char *argv[])
 
   current_line = ibuf_init ();
 
+  cgdbrc_init ();
+
   if (create_and_init_pair () == -1)
     {
       fprintf (stderr, "%s:%d Unable to create PTY pair", __FILE__, __LINE__);
