@@ -24,6 +24,7 @@
 #include "cgdb.h"
 #include "sys_util.h"
 #include "std_list.h"
+#include "kui_term.h"
 
 extern struct tgdb *tgdb;
 
@@ -47,6 +48,7 @@ enum ConfigType
 
 static int command_set_arrowstyle( const char *value );
 static int command_set_focus( const char *value );
+static int command_set_cgdb_mode_key( const char *value );
 static int command_set_winsplit( const char *value );
 static int command_set_timeout( int value );
 static int command_set_timeoutlen( int value );
@@ -64,9 +66,9 @@ static int cgdbrc_set_val (struct cgdbrc_config_option config_option);
 static struct cgdbrc_config_option cgdbrc_config_options[CGDBRC_WRAPSCAN+1] =
 {
   {CGDBRC_ARROWSTYLE, {ARROWSTYLE_SHORT}},
-  {CGDBRC_AUTOSOURCERELOAD, {0}},
+  {CGDBRC_AUTOSOURCERELOAD, {1}},
+  {CGDBRC_CGDB_MODE_KEY, {CGDB_KEY_ESC}},
   {CGDBRC_IGNORECASE, {0}},
-  {CGDBRC_SHORTCUT, {0}},
   {CGDBRC_SHOWTGDBCOMMANDS, {0}},
   {CGDBRC_SYNTAX, {TOKENIZER_LANGUAGE_UNKNOWN}},
   {CGDBRC_TABSTOP, {8}},
@@ -101,12 +103,12 @@ static struct ConfigVariable
     /* autosourcereload */  
     { "autosourcereload", "asr", CONFIG_TYPE_BOOL, &cgdbrc_config_options[CGDBRC_AUTOSOURCERELOAD].variant.int_val },
 
+    /* cgdbmodekey */
+    { "cgdbmodekey", "cgdbmodekey", CONFIG_TYPE_FUNC_STRING, command_set_cgdb_mode_key },
+
     /* ignorecase */
     { "ignorecase", "ic", CONFIG_TYPE_BOOL, &cgdbrc_config_options[CGDBRC_IGNORECASE].variant.int_val },
 
-    /* shortcut   */
-    { "shortcut", "sc", CONFIG_TYPE_BOOL, &cgdbrc_config_options[CGDBRC_SHORTCUT].variant.int_val },
-
     /* showtgdbcommands */
     { "showtgdbcommands", "stc", CONFIG_TYPE_FUNC_BOOL, &command_set_stc },
 
@@ -138,68 +140,100 @@ static struct ConfigVariable
     { "wrapscan", "ws", CONFIG_TYPE_BOOL, &cgdbrc_config_options[CGDBRC_WRAPSCAN].variant.int_val },
 };
 
-static int command_focus_cgdb( void );
-static int command_focus_gdb( void );
-static int command_focus_tty( void );
+static int command_do_tgdbcommand( int param );
 
-static int command_do_bang( void );
-static int command_do_continue( void );
-static int command_do_finish( void );
-static int command_do_focus (void);
-static int command_do_help( void );
-static int command_do_next( void );
-static int command_do_quit( void );
-static int command_do_run( void );
-static int command_do_shell( void );
-static int command_do_step( void );
-static int command_source_reload( void );
+static int command_focus_cgdb( int param );
+static int command_focus_gdb( int param );
+static int command_focus_tty( int param );
 
-static int command_parse_syntax( void );
-static int command_parse_highlight (void);
-static int command_parse_map (void);
-static int command_parse_unmap (void);
+static int command_do_bang( int param );
+static int command_do_focus ( int param );
+static int command_do_help( int param );
+static int command_do_quit( int param );
+static int command_do_shell( int param );
+static int command_source_reload( int param );
 
-typedef int (*action_t)(void);
-static struct commands
+static int command_parse_syntax( int param );
+static int command_parse_highlight ( int param );
+static int command_parse_map ( int param );
+static int command_parse_unmap ( int param );
+
+typedef int (*action_t)( int param );
+typedef struct COMMANDS
 {
-    const char *name, *s_name;
-    /* these functions will return 0 on success and 1 on error.  */
-    /* Should the configuration file processing continue after an error?*/
-    action_t action;
-} COMMANDS[] = {
-    /* keep this stuff sorted, you can use !sort in vi*/
-    /* bang        */ { "bang",        "bang",	    command_do_bang },
-    /* continue    */ { "continue",    "continue",  command_do_continue },
-    /* edit        */ { "edit",        "e",	    command_source_reload },
-    /* finish      */ { "finish",      "finish",    command_do_finish },
-    /* focus      */  { "focus",       "focus",	    command_do_focus },
-    /* help        */ { "help",        "help",	    command_do_help },
-    /* highlight   */ { "highlight",   "hi",	    command_parse_highlight },
-    /* imap	   */ { "imap",	       "im",	    command_parse_map },
-    /* iunmap	   */ { "iunmap",      "iu",	    command_parse_unmap },
-    /* insert      */ { "insert",      "insert",    command_focus_gdb },
-    /* map         */ { "map",         "map",	    command_parse_map },
-    /* next        */ { "next",        "next",	    command_do_next },
-    /* quit        */ { "quit",        "q",	    command_do_quit },
-    /* run         */ { "run",         "run",	    command_do_run },
-    /* shell       */ { "shell",       "sh",	    command_do_shell },
-    /* step        */ { "step",        "step",	    command_do_step },
-    /* syntax      */ { "syntax",      "syntax",    command_parse_syntax },
-    /* unmap       */ { "unmap",       "unm",	    command_parse_unmap },
+	const char *name;
+	/* these functions will return 0 on success and 1 on error.	*/
+	/* Should the configuration file processing continue after an error?*/
+	action_t action;
+	int param;
+} COMMANDS;
+
+COMMANDS commands[] =
+{
+	/* bang			*/ { "bang",		command_do_bang,		0 },
+	/* edit			*/ { "edit",		command_source_reload,	0 },
+	/* edit			*/ { "e",			command_source_reload,	0 },
+	/* focus		*/ { "focus",		command_do_focus,		0 },
+	/* help			*/ { "help",		command_do_help,		0 },
+	/* highlight	        */ { "highlight",	command_parse_highlight,0 },
+	/* highlight	        */ { "hi",			command_parse_highlight,0 },
+	/* imap			*/ { "imap",		command_parse_map,		0 },
+	/* imap			*/ { "im",			command_parse_map,		0 },
+	/* iunmap		*/ { "iunmap",		command_parse_unmap,	0 },
+	/* iunmap		*/ { "iu",			command_parse_unmap,	0 },
+	/* insert		*/ { "insert",		command_focus_gdb,		0 },
+	/* map			*/ { "map",			command_parse_map,		0 },
+	/* quit			*/ { "quit",		command_do_quit,		0 },
+	/* quit			*/ { "q",			command_do_quit,		0 },
+	/* shell		*/ { "shell",		command_do_shell,		0 },
+	/* shell		*/ { "sh",			command_do_shell,		0 },
+	/* syntax		*/ { "syntax",		command_parse_syntax,	0 },
+	/* unmap		*/ { "unmap",		command_parse_unmap,	0 },
+	/* unmap		*/ { "unm",			command_parse_unmap,	0 },
+	/* continue		*/ { "continue",	command_do_tgdbcommand,	TGDB_CONTINUE },
+	/* continue		*/ { "c",			command_do_tgdbcommand,	TGDB_CONTINUE },
+	/* down	         	*/ { "down",			command_do_tgdbcommand,	TGDB_DOWN },
+	/* finish		*/ { "finish",		command_do_tgdbcommand,	TGDB_FINISH },
+	/* finish		*/ { "f",			command_do_tgdbcommand,	TGDB_FINISH },
+	/* next			*/ { "next",		command_do_tgdbcommand,	TGDB_NEXT },
+	/* next			*/ { "n",			command_do_tgdbcommand,	TGDB_NEXT },
+	/* run			*/ { "run",			command_do_tgdbcommand,	TGDB_RUN },
+	/* run			*/ { "r",			command_do_tgdbcommand,	TGDB_RUN },
+	/* kill			*/ { "kill",		command_do_tgdbcommand,	TGDB_KILL },
+	/* kill			*/ { "k",			command_do_tgdbcommand,	TGDB_KILL },
+	/* step			*/ { "step",		command_do_tgdbcommand,	TGDB_STEP },
+	/* step			*/ { "s",			command_do_tgdbcommand,	TGDB_STEP },
+	/* start		*/ { "start",		command_do_tgdbcommand,	TGDB_START },
+	/* up	        	*/ { "up",		command_do_tgdbcommand,	TGDB_UP },
 };
 
-action_t get_command( const char *cmd )
+int command_sort_find(const void *_left_cmd, const void *_right_cmd)
 {
-    /* FIXME: replace with binary search */
-    int i;
-    for ( i = 0; i < (sizeof( COMMANDS )/sizeof( COMMANDS[0] )); ++i ) {
-        if ( strcmp( cmd, COMMANDS[i].name ) == 0 ||
-             strcmp( cmd, COMMANDS[i].s_name ) == 0) {
-            return COMMANDS[i].action;
-        }
-    }
+	COMMANDS *right_cmd = (COMMANDS *)_right_cmd;
+	COMMANDS *left_cmd = (COMMANDS *)_left_cmd;
 
-    return NULL;
+	return strcmp(left_cmd->name,right_cmd->name);
+}
+
+#define COMMANDS_SIZE		(sizeof(COMMANDS))
+#define COMMANDS_COUNT		(sizeof(commands) / sizeof(COMMANDS))
+
+void cgdbrc_init (void)
+{
+	qsort((void *)commands,
+			COMMANDS_COUNT,
+			COMMANDS_SIZE,
+			command_sort_find);
+}
+
+COMMANDS *get_command( const char *cmd )
+{
+	COMMANDS command = { cmd, NULL, 0 };
+
+	return bsearch((void *)&command,(void *)commands,
+					COMMANDS_COUNT,
+					COMMANDS_SIZE,
+					command_sort_find);
 }
 
 struct ConfigVariable* get_variable( const char *variable ) 
@@ -234,6 +268,31 @@ command_set_arrowstyle (const char *value)
   return cgdbrc_set_val (option);
 }
 
+int
+command_set_cgdb_mode_key (const char *value)
+{
+  struct cgdbrc_config_option option;
+  option.option_kind = CGDBRC_CGDB_MODE_KEY;
+
+  if (value) {
+    /* If the user typed in a single key, use it. */
+    if (strlen (value) == 1) {
+      option.variant.int_val = value[0];
+    } else {
+      /* The user may have typed in a keycode. (e.g. <Esc>)
+       * attempt to translate it. */
+      int key = kui_term_get_cgdb_key_from_keycode (value);
+      if (key == -1)
+        return -1;
+      option.variant.int_val = key;
+    }
+  } else {
+    return -1;
+  }
+
+  return cgdbrc_set_val (option);
+}
+
 static int 
 command_set_stc (int value)
 {
@@ -375,33 +434,34 @@ command_set_syntax_type (const char *value)
   return 0;
 }
 
-int command_focus_cgdb( void )
+int command_focus_cgdb( int param )
 {
     if_set_focus( CGDB );
     return 0;
 }
 
-int command_focus_gdb( void )
+int command_focus_gdb( int param )
 {
     if_set_focus( GDB );
     return 0;
 }
 
-int command_focus_tty( void )
+int command_focus_tty( int param )
 {
     if_set_focus( TTY );
     return 0;
 }
 
-int command_do_bang( void )
+int command_do_bang( int param )
 {
     return 0;
 }
 
-int command_do_continue( void )
+int command_do_tgdbcommand( int param )
 {
     tgdb_request_ptr request_ptr;
-    request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_CONTINUE);
+
+    request_ptr = tgdb_request_run_debugger_command (tgdb, param );
     if (!request_ptr)
         return -1;
     handle_request (tgdb, request_ptr);
@@ -409,19 +469,7 @@ int command_do_continue( void )
     return 0;
 }
 
-int command_do_finish( void )
-{
-    tgdb_request_ptr request_ptr;
-    request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_FINISH);
-    if (!request_ptr)
-        return -1;
-    handle_request (tgdb, request_ptr);
-
-    return 0;
-}
-
-int 
-command_do_focus (void)
+int command_do_focus ( int param )
 {
   int token = yylex ();
   const char *value;
@@ -431,35 +479,24 @@ command_do_focus (void)
   value = get_token ();
 
   if (strcasecmp (value, "cgdb") == 0)
-    command_focus_cgdb();
+    command_focus_cgdb(0);
   else if (strcasecmp (value, "gdb") == 0)
-    command_focus_gdb();
+    command_focus_gdb(0);
   else if (strcasecmp (value, "tty") == 0)
-    command_focus_tty();
+    command_focus_tty(0);
   else
     return 1;
 
   return 0;
 }
 
-int command_do_help( void )
+int command_do_help( int param )
 {
     if_display_help();
     return 0;
 }
 
-int command_do_next( void )
-{
-    tgdb_request_ptr request_ptr;
-    request_ptr = tgdb_request_run_debugger_command  (tgdb, TGDB_NEXT);
-    if (!request_ptr)
-        return -1;
-    handle_request (tgdb, request_ptr);
-
-    return 0;
-}
-
-int command_do_quit( void )
+int command_do_quit( int param )
 {
     /* FIXME: Test to see if debugged program is still running */
     cleanup();
@@ -467,35 +504,12 @@ int command_do_quit( void )
     return 0;
 }
 
-int command_do_run( void )
-{
-    /* FIXME: see if there are any other arguments to pass to the run command */
-    tgdb_request_ptr request_ptr;
-    request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_RUN);
-    if (!request_ptr)
-        return -1;
-    handle_request (tgdb, request_ptr);
-
-    return 0;
-}
-
-int command_do_shell( void )
+int command_do_shell( int param )
 {
     return run_shell_command(NULL);
 }
 
-int command_do_step( void )
-{
-    tgdb_request_ptr request_ptr;
-    request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_STEP);
-    if (!request_ptr)
-        return -1;
-    handle_request (tgdb, request_ptr);
-
-    return 0;
-}
-
-int command_source_reload( void )
+int command_source_reload( int param )
 {
     struct sviewer *sview = if_get_sview ();
 
@@ -512,7 +526,7 @@ int command_source_reload( void )
     return 0;
 }
 
-int command_parse_syntax( void )
+int command_parse_syntax( int param )
 {
   /* This is something like: 
      :syntax
@@ -543,19 +557,21 @@ int command_parse_syntax( void )
   return 0;
 }
 
-static int 
-command_parse_highlight (void)
+static int command_parse_highlight ( int param )
 {
   return hl_groups_parse_config (hl_groups_instance);
 }
 
 extern struct kui_map_set *kui_map, *kui_imap;
-static int
-command_parse_map (void)
+
+static int command_parse_map ( int param )
 {
   struct kui_map_set *kui_map_choice;
   int key, value, val;
   char *key_token;
+  extern int enter_map_id;
+
+  enter_map_id = 1;
 
   if (strcmp (get_token (), "map") == 0)
     kui_map_choice = kui_map;
@@ -564,13 +580,17 @@ command_parse_map (void)
 
   key = yylex ();
   if (key != IDENTIFIER)
+  {
+    enter_map_id = 0;
     return -1;
+  }
   key_token = cgdb_strdup (get_token ());
 
   value = yylex ();
   if (value != IDENTIFIER)
   {
-    xfree (key_token);
+    free (key_token);
+    enter_map_id = 0;
     return -1;
   }
 
@@ -578,18 +598,23 @@ command_parse_map (void)
   if (val == -1)
   {
     free (key_token);
+    enter_map_id = 0;
     return -1;
   }
 
+  enter_map_id = 0;
+
   return 0;
 }
 
-static int
-command_parse_unmap (void)
+static int command_parse_unmap ( int param )
 {
   struct kui_map_set *kui_map_choice;
   int key, val;
   char *key_token;
+  extern int enter_map_id;
+
+  enter_map_id = 1;
 
   if ((strcmp (get_token (), "unmap") == 0) || 
       (strcmp (get_token (), "unm") == 0))
@@ -599,16 +624,22 @@ command_parse_unmap (void)
 
   key = yylex ();
   if (key != IDENTIFIER)
+  {
+    enter_map_id = 0;
     return -1;
+  }
   key_token = cgdb_strdup (get_token ());
 
   val = kui_ms_deregister_map (kui_map_choice, key_token);
   if (val == -1)
   {
     free (key_token);
+    enter_map_id = 0;
     return -1;
   }
 
+  enter_map_id = 0;
+
   return 0;
 }
 
@@ -776,9 +807,9 @@ int command_parse_string( const char *buffer )
     } break;
 
     case IDENTIFIER: {
-        action_t action = get_command( get_token() );
-        if ( action ) {
-            action();
+        COMMANDS *command = get_command( get_token() );
+        if ( command ) {
+            command->action(command->param);
             rv = 0;
         } else {
             rv = 1;
