@@ -110,16 +110,32 @@ static struct winsize screen_size;	/* Screen size */
 
 struct filedlg *fd;		/* The file dialog structure */
 
-static char regex_line[MAX_LINE];	/* The regex the user enters */
-static int regex_line_pos;	/* The index into the current regex */
-static int regex_search;	/* Currently searching text ? */
-static int regex_direction;	/* Direction to search */
+/* The regex the user is entering */
+static struct ibuf *regex_cur = NULL;
+
+/* The last regex the user searched for */
+static struct ibuf *regex_last = NULL;
+
+/* Direction to search */
+static int regex_direction_cur;
+static int regex_direction_last;
+
+/* The position of the source file, before regex was applied. */
+static int orig_line_regex; 
 
 static char last_key_pressed = 0;	/* Last key user entered in cgdb mode */
 
-static char cur_com_line[MAX_LINE];	/* The line number the user entered */
-static int cur_com_pos;		/* The index into the current line */
-static int display_command;	/* Currently going to a line number */
+/* The cgdb status bar command */
+static struct ibuf *cur_sbc = NULL;
+
+enum StatusBarCommandKind {
+    /* This is represented by a : */
+    SBC_NORMAL,
+    /* This is set when a regular expression is being entered */
+    SBC_REGEX
+}; 
+
+static enum StatusBarCommandKind sbc_kind;
 
 /* --------------- */
 /* Local Functions */
@@ -332,28 +348,31 @@ update_status_win (void)
     mvwprintw (status_win, 0, WIDTH - 1, "*");
   else if (focus == TTY && tty_win_on)
     mvwprintw (tty_status_win, 0, WIDTH - 1, "*");
-  else if (focus == CGDB)
+  else if (focus == CGDB || focus == CGDB_STATUS_BAR)
     mvwprintw (status_win, 0, WIDTH - 1, " ");
   wattroff (status_win, attr);
   if (tty_win_on)
     wattroff (tty_status_win, attr);
 
   /* Print the regex that the user is looking for Forward */
-  if (regex_search && regex_direction)
+  if (focus == CGDB_STATUS_BAR && sbc_kind == SBC_REGEX && regex_direction_cur)
     {
-      if_display_message ("/", WIDTH - 1, "%s", regex_line);
+      if_display_message ("/", WIDTH - 1, "%s", ibuf_get (regex_cur));
       curs_set (1);
     }
   /* Regex backwards */
-  else if (regex_search)
+  else if (focus == CGDB_STATUS_BAR && sbc_kind == SBC_REGEX)
     {
-      if_display_message ("?", WIDTH - 1, "%s", regex_line);
+      if_display_message ("?", WIDTH - 1, "%s", ibuf_get (regex_cur));
       curs_set (1);
     }
-  /* Line number search */
-  else if (display_command)
+  /* A colon command typed at the status bar */
+  else if (focus == CGDB_STATUS_BAR && sbc_kind == SBC_NORMAL)
     {
-      if_display_message (":", WIDTH - 1, "%s", cur_com_line);
+      char *command = ibuf_get (cur_sbc);
+      if (!command)
+        command = "";
+      if_display_message (":", WIDTH - 1, "%s", command);
       curs_set (1);
     }
   /* Default: Current Filename */
@@ -773,85 +792,21 @@ signal_handler (int signo)
     }
 }
 
-/* if_get_command: Gets a command from the user
- * -----------------
- */
 static void
-if_get_command (struct sviewer *sview)
+if_run_command (struct sviewer *sview, struct ibuf *ibuf_command)
 {
-  int c;
-  cur_com_pos = 0;
-  cur_com_line[cur_com_pos] = '\0';
-  display_command = 1;
-  if_draw ();
-
-  while ((c = wgetch (sview->win)) != ERR)
-    {
-      if (cur_com_pos == (MAX_LINE - 1)
-	  && !(c == CGDB_KEY_ESC || c == 8 || c == 127))
-	continue;
-
-      /* Quit the search if the user hit escape */
-      if (c == 27)
-	{
-	  cur_com_pos = 0;
-	  cur_com_line[cur_com_pos] = '\0';
-	  display_command = 0;
-	  if_draw ();
-	  return;
-	}
-
-      /* If the user hit enter, then a successful regex has been recieved */
-      if (c == '\r' || c == '\n')
-	{
-	  cur_com_line[cur_com_pos] = '\0';
-	  display_command = 0;
-	  return;
-	}
-
-      /* If the user hit backspace or delete remove a char */
-      if (CGDB_BACKSPACE_KEY (c) || c == 8 || c == 127)
-	{
-	  if (cur_com_pos > 0)
-	    --cur_com_pos;
-
-	  cur_com_line[cur_com_pos] = '\0';
-	  if_draw ();
-
-	  update_status_win ();
-	  wrefresh (status_win);
-	  continue;
-	}
-
-      /* Add a char, search and draw */
-      cur_com_line[cur_com_pos++] = c;
-      cur_com_line[cur_com_pos] = '\0';
-      if_draw ();
-      update_status_win ();
-      wrefresh (status_win);
-    }
-
-  /* Finished */
-  display_command = 0;
-  if_draw ();
-}
-
-static void
-if_run_command (struct sviewer *sview)
-{
-  /* Get a command and then try to process it */
-  if_get_command (sview);
+  char *command = ibuf_get (ibuf_command);
 
   /* refresh and return if the user entered no data */
-  if (cur_com_pos == 0)
+  if (ibuf_length (ibuf_command) == 0)
     {
       if_draw ();
       return;
     }
 
-  if (command_parse_string (cur_com_line))
+  if (command_parse_string (command))
     {
-      if_display_message ("Unknown command: ", 0, "%s", cur_com_line);
+      if_display_message ("Unknown command: ", 0, "%s", command);
     }
   else
     {
@@ -861,94 +816,6 @@ if_run_command (struct sviewer *sview)
   if_draw ();
 }
 
-/* capture_regex: Captures a regular expression from the user.
- * ---------------
- *
- *   sview:  Source viewer object
- *
- *  Side Effect: 
- *
- *  regex_line: The regex the user has entered.
- *  regex_line_pos: The next available index into regex_line.
- *
- * Return Value: 0 if user gave a regex, otherwise 1.
- */
-static int
-capture_regex (struct sviewer *sview)
-{
-  int c;
-  int orig_line; 
-  int regex_icase = cgdbrc_get (CGDBRC_IGNORECASE)->variant.int_val;
-
-  if (!sview || !sview->cur)
-    return 0; 
-  
-  orig_line = sview->cur->sel_line;
-  /* Initialize the function for finding a regex and tell user */
-  regex_search = 1;
-  regex_line_pos = 0;
-  regex_line[regex_line_pos] = '\0';
-  if_draw ();
-
-  while ((c = wgetch (sview->win)) != ERR)
-    {
-      if (regex_line_pos == (MAX_LINE - 1)
-	  && !(c == CGDB_KEY_ESC || c == 8 || c == 127))
-	continue;
-
-      /* Quit the search if the user hit escape */
-      if (c == 27)
-	{
-	  regex_line_pos = 0;
-	  *regex_line = '\0';
-	  regex_search = 0;
-	  source_search_regex (sview, regex_line, 2, regex_direction,
-			       regex_icase);
-	  sview->cur->sel_line = orig_line;
-	  if_draw ();
-	  return 1;
-	}
-
-      /* If the user hit enter, then a successful regex has been recieved */
-      if (c == '\r' || c == '\n')
-	{
-	  regex_line[regex_line_pos] = '\0';
-	  regex_search = 0;
-	  source_search_regex (sview, regex_line, 2, regex_direction,
-			       regex_icase);
-	  if_draw ();
-	  return 0;
-	}
-
-      /* If the user hit backspace or delete remove a char */
-      if (CGDB_BACKSPACE_KEY (c) || c == 8 || c == 127)
-	{
-	  if (regex_line_pos > 0)
-	    --regex_line_pos;
-
-	  regex_line[regex_line_pos] = '\0';
-	  source_search_regex (sview, regex_line, 1, regex_direction,
-			       regex_icase);
-	  if_draw ();
-
-	  update_status_win ();
-	  continue;
-	}
-
-      /* Add a char, search and draw */
-      regex_line[regex_line_pos++] = c;
-      regex_line[regex_line_pos] = '\0';
-      source_search_regex (sview, regex_line, 1, regex_direction,
-			   regex_icase);
-      if_draw ();
-      update_status_win ();
-    }
-
-  /* Finished */
-  regex_search = 0;
-  return 0;
-}
-
 /* tty_input: Handles user input to the tty I/O window.
  * ----------
  *
@@ -1012,6 +879,39 @@ gdb_input (int key)
     case CGDB_KEY_F12:
       scr_end (gdb_win);
       break;
+#if 0
+    /* I would like to add better support for control-l in the GDB
+     * window, but this patch didn't make me happy enough to release it.
+     * The problem is, when it clears the screen, it adds a lot of 
+     * whitespace in the buffer. If you hit page-up to look back in
+     * the buffer, it's visible. This is really unacceptable.
+     *
+     * The approach I believe I would like to take with this, is to
+     * have the GDB window behave more like the terminal. That is,
+     * have GDB start at the top line, and move down as input 
+     * becomes available. Then, when you hit ctrl-l, you just move
+     * the prompt to the top line. */
+    case CGDB_KEY_CTRL_L: 
+    {
+      int height = get_gdb_height(), i;
+
+      /* Allocate and print enough newlines to clear the gdb buffer. */
+      char *buf = (char*)cgdb_malloc(sizeof(char*)*height);
+      for (i = 0; i < height-1; ++i) {
+        buf[i] = '\n';
+      }
+      buf[i] = '\0';
+      if_print(buf);
+      free(buf);
+
+      /* Sneaky return 1 here. Basically, this allows tricks readline to think
+       * that gdb did not handle the Ctrl-l. That way readline will also handle
+       * it. Because readline uses TERM=dumb, that means that it will clear a 
+       * single line and put out the prompt. */
+      return 1;
+      break;
+    }
+#endif
     default:
       return 1;
     }
@@ -1021,6 +921,137 @@ gdb_input (int key)
   return 0;
 }
 
+/**
+ * Capture a regular expression from the user, one key at a time.
+ * This modifies the global variables regex_cur and regex_last.
+ *
+ * \param sview
+ * The source viewer.
+ *
+ * \return
+ * 0 if user gave a regex, otherwise 1.
+ */
+static int
+status_bar_regex_input (struct sviewer *sview, int key)
+{
+  int regex_icase = cgdbrc_get (CGDBRC_IGNORECASE)->variant.int_val;
+
+  /* Flag to indicate we're done with regex mode, need to switch back */
+  int done = 0;
+
+  /* Recieve a regex from the user. */
+  switch (key) {
+    case '\r':
+    case '\n':
+    case CGDB_KEY_CTRL_M:
+      /* Save for future searches via 'n' or 'N' */
+      if (regex_last != NULL) {
+        ibuf_free (regex_last);
+      }
+      regex_last = ibuf_dup (regex_cur);
+      regex_direction_last = regex_direction_cur;
+      source_search_regex (sview, ibuf_get (regex_last), 2,
+          regex_direction_last, regex_icase);
+      if_draw ();
+      done = 1;
+      break;
+    case 8:
+    case 127:
+      /* Backspace or DEL key */
+      if (ibuf_length (regex_cur) == 0) {
+        done = 1;
+      } else {
+        ibuf_delchar (regex_cur);
+        source_search_regex (sview, ibuf_get (regex_cur), 1,
+            regex_direction_cur, regex_icase);
+        if_draw ();
+        update_status_win ();
+      }
+      break;
+    default:
+      if (kui_term_is_cgdb_key (key)) {
+        const char *keycode = kui_term_get_keycode_from_cgdb_key (key);
+        int length = strlen (keycode), i;
+        for (i = 0; i < length; i++)
+          ibuf_addchar (regex_cur, keycode[i]);
+      } else {
+        ibuf_addchar (regex_cur, key);
+      }
+      source_search_regex (sview, ibuf_get (regex_cur), 1,
+          regex_direction_cur, regex_icase);
+      if_draw ();
+      update_status_win ();
+  };
+
+  if (done) {
+    ibuf_free (regex_cur);
+    regex_cur = NULL;
+    if_set_focus (CGDB);
+  }
+
+  return 0;
+}
+
+static int
+status_bar_normal_input (int key)
+{
+  /* Flag to indicate we're done with status mode, need to switch back */
+  int done = 0;
+
+  /* The goal of this state is to recieve a command from the user. */
+  switch (key) {
+    case '\r':
+    case '\n':
+    case CGDB_KEY_CTRL_M:
+      /* Found a command */
+      if_run_command (src_win, cur_sbc);
+      done = 1;
+      break;
+    case 8:
+    case 127:
+      /* Backspace or DEL key */
+      if (ibuf_length (cur_sbc) == 0) {
+        done = 1;
+      } else {
+        ibuf_delchar (cur_sbc);
+        update_status_win ();
+      }
+      break;
+    default:
+      if (kui_term_is_cgdb_key (key)) {
+        const char *keycode = kui_term_get_keycode_from_cgdb_key (key);
+        int length = strlen (keycode), i;
+        for (i = 0; i < length; i++)
+          ibuf_addchar (cur_sbc, keycode[i]);
+      } else {
+        ibuf_addchar (cur_sbc, key);
+      }
+      update_status_win ();
+      break;
+  };
+
+  if (done) {
+    ibuf_free (cur_sbc);
+    cur_sbc = NULL;
+    if_set_focus (CGDB);
+  }
+
+  return 0;
+}
+
+static int
+status_bar_input (struct sviewer *sview, int key)
+{
+  switch (sbc_kind) {
+    case SBC_NORMAL:
+      return status_bar_normal_input (key);
+    case SBC_REGEX:
+      return status_bar_regex_input (sview, key);
+  };
+
+  return -1;
+}
+
 /**
  * toggle a breakpoint
  * 
@@ -1047,7 +1078,7 @@ toggle_breakpoint (struct sviewer *sview, enum tgdb_breakpoint_action t)
 
   /* Get filename (strip path off -- GDB is dumb) */
   path = strrchr (sview->cur->path, '/') + 1;
-  if ((int) path == 1)
+  if (path == NULL + 1)
     path = sview->cur->path;
 
   /* delete an existing breakpoint */
@@ -1072,8 +1103,6 @@ toggle_breakpoint (struct sviewer *sview, enum tgdb_breakpoint_action t)
 static void
 source_input (struct sviewer *sview, int key)
 {
-  int shortcut_option = cgdbrc_get (CGDBRC_SHORTCUT)->variant.int_val;
-
   switch (key)
     {
     case CGDB_KEY_UP:
@@ -1132,15 +1161,13 @@ source_input (struct sviewer *sview, int key)
     case 'o':
       /* Causes file dialog to be opened */
       {
+        extern int kui_input_acceptable;
+        kui_input_acceptable = 0;
 	tgdb_request_ptr request_ptr;
 	request_ptr = tgdb_request_inferiors_source_files (tgdb);
 	handle_request (tgdb, request_ptr);
       }
       break;
-    case ':':
-      /* Allows user to go to a line number */
-      if_run_command (sview);
-      return;
     case ' ':
       {
 	enum tgdb_breakpoint_action t = TGDB_BREAKPOINT_ADD;
@@ -1158,40 +1185,6 @@ source_input (struct sviewer *sview, int key)
     }
 
   /* Some extended features that are set by :set sc */
-  if (shortcut_option)
-    {
-      tgdb_request_ptr request_ptr = NULL;
-      switch (key)
-	{
-	case 'r':
-	  request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_RUN);
-	  break;
-	case 'n':
-	  request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_NEXT);
-	  break;
-	case 's':
-	  request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_STEP);
-	  break;
-	case 'c':
-	  request_ptr =
-	    tgdb_request_run_debugger_command (tgdb, TGDB_CONTINUE);
-	  break;
-	case 'f':
-	  request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_FINISH);
-	  break;
-	case 'u':
-	  request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_UP);
-	  break;
-	case 'd':
-	  request_ptr = tgdb_request_run_debugger_command (tgdb, TGDB_DOWN);
-	  break;
-	default:
-	  break;
-	}
-      if (request_ptr)
-	handle_request (tgdb, request_ptr);
-    }
-
   if_draw ();
 }
 
@@ -1286,15 +1279,30 @@ int
 internal_if_input (int key)
 {
   int regex_icase = cgdbrc_get (CGDBRC_IGNORECASE)->variant.int_val;
-  int shortcut_option = cgdbrc_get (CGDBRC_SHORTCUT)->variant.int_val;
+  /* Normally, CGDB_KEY_ESC, but can be configured by the user */
+  int cgdb_mode_key = cgdbrc_get (CGDBRC_CGDB_MODE_KEY)->variant.int_val;
 
-  /* The ESC Key, puts the debugger into command mode */
-  if (focus != CGDB && key == CGDB_KEY_ESC)
+  /* The cgdb mode key, puts the debugger into command mode */
+  if (focus != CGDB && key == cgdb_mode_key)
     {
+      /* Depending on which cgdb was in, it can free some memory here that
+       * it was previously using. */
+      if (focus == CGDB_STATUS_BAR && sbc_kind == SBC_NORMAL) {
+        ibuf_free (cur_sbc);
+        cur_sbc = NULL;
+      } else if (focus == CGDB_STATUS_BAR && sbc_kind == SBC_REGEX) {
+        ibuf_free (regex_cur);
+        regex_cur = NULL;
+        free(src_win->cur->buf.cur_line);
+        src_win->cur->buf.cur_line = NULL;
+        src_win->cur->sel_rline = orig_line_regex;
+	src_win->cur->sel_line = orig_line_regex;
+      }
       if_set_focus (CGDB);
       return 0;
     }
-  else if (key == CGDB_KEY_ESC)
+  /* If you are already in cgdb mode, the cgdb mode key does nothing */
+  else if (key == cgdb_mode_key)
     return 0;
 
   /* Check for global keystrokes */
@@ -1309,25 +1317,39 @@ internal_if_input (int key)
 	case 'I':
 	  if_set_focus (TTY);
 	  return 0;
+        case ':':
+          /* Set the type of the command the user is typing in the status bar */
+          sbc_kind = SBC_NORMAL;
+          if_set_focus (CGDB_STATUS_BAR);
+          /* Since the user is about to type in a command, allocate a buffer 
+           * in which this command can be stored. */
+	  cur_sbc = ibuf_init ();
+          return 0;
 	case '/':
 	case '?':
-	  regex_direction = ('/' == key);
+          if (src_win->cur != NULL) {
+            regex_cur = ibuf_init ();
+	    regex_direction_cur = ('/' == key);
+            orig_line_regex = src_win->cur->sel_line;
 
-	  /* Capturing regular expressions */
-	  source_search_regex_init (src_win);
-	  capture_regex (src_win);
+            sbc_kind = SBC_REGEX;
+            if_set_focus (CGDB_STATUS_BAR);
+
+	    /* Capturing regular expressions */
+	    source_search_regex_init (src_win);
+
+            /* Initialize the function for finding a regex and tell user */
+            if_draw ();
+          }
 	  return 0;
 	case 'n':
-	  if (!shortcut_option)
-	    {
-	      source_search_regex (src_win, regex_line, 2, regex_direction,
-				   regex_icase);
-	      if_draw ();
-	    }
+	  source_search_regex (src_win, ibuf_get (regex_last), 2,
+              regex_direction_last, regex_icase);
+	  if_draw ();
 	  break;
 	case 'N':
-	  source_search_regex (src_win, regex_line, 2, !regex_direction,
-			       regex_icase);
+	  source_search_regex (src_win, ibuf_get (regex_last), 2,
+              !regex_direction_last, regex_icase);
 	  if_draw ();
 	  break;
 	case 'T':
@@ -1439,10 +1461,9 @@ internal_if_input (int key)
 	    return 0;
 	  }
       }
-    default:
-      /* Focus is screwed up, fix it */
-      fprintf (stderr, "KEYBOARD ERROR\r\n");
       return 0;
+    case CGDB_STATUS_BAR:
+      return status_bar_input (src_win, key);
     }
 
   /* Never gets here */
@@ -1604,6 +1625,9 @@ if_set_focus (Focus f)
       focus = f;
       if_draw ();
       break;
+    case CGDB_STATUS_BAR:
+      focus = f;
+      if_draw ();
     default:
       return;
     }
