@@ -5,12 +5,15 @@
 #endif /* HAVE_SYS_IOCTL_H */
 
 int 
-tty_cbreak (int fd)
+tty_cbreak (int fd, struct termios *orig)
 {
   struct termios buf;
    
   if (tcgetattr (fd, &buf) < 0)
     return -1;
+
+  /* Save the original state, for resetting later */
+  *orig = buf;
       
   buf.c_lflag &= ~(ECHO | ICANON);
   buf.c_iflag &= ~(ICRNL | INLCR);
@@ -68,6 +71,25 @@ tty_off_xon_xoff (int fd)
   return 0;
 }
 
+int 
+tty_set_echo (int fd, int echo_on)
+{
+  struct termios buf;
+
+  if (tcgetattr (fd, &buf) < 0)
+    return -1;
+
+  if (echo_on == 1)
+    buf.c_lflag |= ECHO;
+  else if (echo_on == 0)
+    buf.c_lflag &= ~ECHO;
+
+  if (tcsetattr (fd, TCSAFLUSH, &buf) < 0)
+    return -1;
+
+  return 0;
+}
+
 int 
 tty_get_attributes (int fd, struct termios *buf)
 {
